
#include <iostream>

#include <Wt/WText>
#include <Wt/WVBoxLayout>
#include <Wt/WHBoxLayout>
#include <Wt/WGridLayout>
#include <Wt/WContainerWidget>
#include <Wt/WImage>

#include "GWt_computingBlock.h"
//#include "GWt_softwarePanel.h"
#include "GWt_machine.h"

using namespace std;

GWt_computingBlock::GWt_computingBlock(Wt::WComboBox* first,Wt::WComboBox* last,Wt::WComboBox* soft,Wt::WContainerWidget* buttonPanel, computingBlock* block,GWt_machine* machine)
:WContainerWidget(),
firstElement_(first),
lastElement_(last),
softElement_(soft),
cpblock_(block),
UImachine_(machine)
{
  Wt::WGridLayout* mainContainerLayout = new Wt::WGridLayout();
  mainContainerLayout->setContentsMargins(0,0,0,0);

  Wt::WContainerWidget* labelAndComboContainer = new Wt::WContainerWidget();
  Wt::WVBoxLayout* labelAndComboContainerLayout = new Wt::WVBoxLayout();
  labelAndComboContainerLayout->setContentsMargins(0,0,0,0);

  Wt::WContainerWidget* labelAndComboInsideContainer = new Wt::WContainerWidget();
  Wt::WHBoxLayout* labelAndComboInsideContainerLayout = new Wt::WHBoxLayout();
  labelAndComboInsideContainerLayout->setContentsMargins(0,0,0,0);
  
  labelAndComboInsideContainerLayout->addWidget(new Wt::WText("from&nbsp;:&nbsp;"));
  labelAndComboInsideContainerLayout->addWidget(firstElement_);
  labelAndComboInsideContainerLayout->addWidget(new Wt::WText("to&nbsp;:&nbsp;"));
  labelAndComboInsideContainerLayout->addWidget(lastElement_);
  labelAndComboInsideContainerLayout->addWidget(softElement_);
  
    // add buttons
  labelAndComboInsideContainerLayout->addWidget(buttonPanel);
  labelAndComboInsideContainer->setLayout(labelAndComboInsideContainerLayout);
  labelAndComboContainerLayout->addWidget(labelAndComboInsideContainer);
  labelAndComboContainerLayout->addStretch (10);
  labelAndComboContainer->setLayout(labelAndComboContainerLayout);
  
  // add errors and warning layout
  warningsContainer_ = new Wt::WContainerWidget();
  errorsContainer_ = new Wt::WContainerWidget();
  
  Wt::WContainerWidget* warningsAndErrorsContainer = new Wt::WContainerWidget();
  Wt::WVBoxLayout* vWarningsAndErrorsLayoutContainer = new Wt::WVBoxLayout();
  
  Wt::WHBoxLayout* hLayoutContainerWarnings = new Wt::WHBoxLayout();
  Wt::WHBoxLayout* hLayoutContainerErrors = new Wt::WHBoxLayout();
  
  
  warningsLabel_ = new Wt::WText("",XHTMLText);
  errorsLabel_ = new Wt::WText("",XHTMLText);
  hLayoutContainerWarnings->addWidget(warningsLabel_);
  hLayoutContainerErrors->addWidget(errorsLabel_);

  warningsAndErrorsContainer->addWidget(warningsLabel_);
  warningsAndErrorsContainer->addWidget(errorsLabel_);
  
  
  // set Layouts
  warningsContainer_->setLayout(hLayoutContainerWarnings);
  errorsContainer_->setLayout(hLayoutContainerErrors);
  
  vWarningsAndErrorsLayoutContainer->addWidget(errorsContainer_);
  vWarningsAndErrorsLayoutContainer->addWidget(warningsContainer_);
  vWarningsAndErrorsLayoutContainer->addStretch (100);
  
  warningsAndErrorsContainer->setLayout(vWarningsAndErrorsLayoutContainer);
  
  // add label and Combo containers
  mainContainerLayout->addWidget(labelAndComboContainer,0,0);
  // add warnings and errors containers
  mainContainerLayout->addWidget(warningsAndErrorsContainer,1,0);
  
  // manage margins
  vWarningsAndErrorsLayoutContainer->setContentsMargins(20,0,0,0);
  hLayoutContainerWarnings->setContentsMargins(0,0,0,0);
  hLayoutContainerErrors->setContentsMargins(0,0,0,0);
  
  setLayout(mainContainerLayout);
  warningsContainer_->setStyleClass("warningsContainer");
  errorsContainer_->setStyleClass("errorsContainer");
  
  // resize combo
  firstElement_->setMinimumSize(80,12);
  lastElement_->setMinimumSize(80,12);
  softElement_->setMinimumSize(80,12);
  
  firstElement_->activated().connect(this,&GWt_computingBlock::changeFirstElement);
  lastElement_->activated().connect(this,&GWt_computingBlock::changeLastElement);
//  lastElement->activated().connect(softwarePanel,&GWt_softwarePanel::updateSections);
  softElement_->activated().connect(this,&GWt_computingBlock::setSoftware);
  this->redraw_.connect(UImachine_,&GWt_machine::displayWidgetComputing);
  
  manageWarningsAndErrors();
}


void GWt_computingBlock::setFirstElementCurrentSelection_deprecated(Wt::WString txt){
    for (int a=0; a< firstElement_->count(); a++) {
        if (firstElement_->itemText(a) == txt) {
            firstElement_->setCurrentIndex (a);
            return;
        }
    }
}

void GWt_computingBlock::setLastElementCurrentSelection_deprecated(Wt::WString txt){
    for (int a=0; a< lastElement_->count(); a++) {
        if (lastElement_->itemText(a) == txt) {
            lastElement_->setCurrentIndex (a);
            return;
        }
    }
}

void GWt_computingBlock::setSoftwareElementCurrentSelection_deprecated(Wt::WString txt){
  for (int a=0; a< softElement_->count(); a++) {
    if (softElement_->itemText(a) == txt) {
      softElement_->setCurrentIndex (a);
      return;
    }
  }
}


void GWt_computingBlock::manageWarningsAndErrors(){
    if (warningsLabel_->text() == "") {
        warningsContainer_->hide();
    } else {
        warningsContainer_->show();
    }
    if (errorsLabel_->text() == "") {
        errorsContainer_->hide();
    } else {
        errorsContainer_->show();
    }

}


 void GWt_computingBlock::setSoftware()
{

UImachine_->getMachineControler()->setSoftware(softElement_->currentText().toUTF8(), cpblock_);
  //  UIsector_->getBeamLineWidget()->buildBeamLineWidget();
}


void GWt_computingBlock::changeFirstElement()
{
  if (!cpblock_) return;
  if (!UImachine_) return;

  UImachine_->getMachineControler()->setFirstElementOfBlock(cpblock_, firstElement_->currentIndex());
  redraw_.emit();
}

void GWt_computingBlock::changeLastElement()
{
  if (!cpblock_) return;
  if (!UImachine_) return;

  UImachine_->getMachineControler()->setLastElementOfBlock(cpblock_, lastElement_->currentIndex());
  redraw_.emit();
}
